function z = generate_diffusion_MG(S, S0, vol, rate, beta)

%  Generator for the chain that approximates
%  diffusion: dS/S = rate  dt + vol (S/S0)^beta dW

N = size(S, 2);       % number of points in the state-space
L(1,:) = 0;           % absorbing boundaries
L(N,:) = 0;

for x = 2:N-1
    dSp = S(x+1)-S(x);  
    dSm  = S(x-1)-S(x);
    muS = rate*S(x);       
    volS = vol*(S(x)/S0)^beta*S(x);
    A = [dSp dSm; dSp^2 dSm^2];
    b = [muS; volS^2];
    ret = A\b;
    L(x,x+1) = ret(1);
    L(x,x-1) = ret(2);
    L(x,x)   = -(L(x,x-1)+L(x,x+1));
end

z = L;


