% 5 important points: s1<s2<s3<s4<s5
% The final grid is consisted of 3 non-uniform grid with densities
% (d1, u1), (d2, u2) and (d3, u3)
% N is the total number grid points

% Schematic picture:      |---------|---------|----------|-----------|
%                         s1        s2        s3        s4          s5

function z = generate_grid_for_double_barrier(N, s1, s2, s3, s4, s5, ...
                                                 d1, u1, ...
                                                 d2, u2, ...
                                                 d3, u3)
N1              = ceil(N/3);   
N3              = N1;
N2              = N-N1-N3;

%----- important grid points
s2_to_s3 = (s2+s3)/2;
s3_to_s4 = (s3+s4)/2;

%  The edges of each sub-grid are overlapped,
%  that's why we have added 2 extra points.
grid1 = generate_non_uniform_grid(N1+1, s1,       s2_to_s3, s2, ceil(N1/2), d1, u1); 
grid2 = generate_non_uniform_grid(N2,   s2_to_s3, s3_to_s4, s3, ceil(N2/2), d2, u2); 
grid3 = generate_non_uniform_grid(N3+1, s3_to_s4, s5,       s4, ceil(N3/2), d3, u3); 

s_grid(1:N1+1)     = grid1;
s_grid(N1+1:N1+N2) = grid2;
s_grid(N1+N2:N)    = grid3;

z = s_grid;
%----- end of generate_grid_for_double_barrier


function z = generate_non_uniform_grid(nx, min_x, max_x, mid_x, mid_x_index, density_lower, density_upper)
c1 = asinh( (min_x-mid_x)/density_lower );
c2 = asinh( (max_x-mid_x)/density_upper );

nx_lower  = mid_x_index;
nx_upper  = nx - nx_lower + 1;

x_lower   = generate_uniform_grid(nx_lower, 0.0, 1.0);
res_lower = mid_x + density_lower * sinh(c1 * (1-x_lower) );

x_upper   = generate_uniform_grid(nx_upper, 0.0, 1.0);
res_upper = mid_x + density_upper * sinh(c2 * x_upper);

res(1:nx_lower)  = res_lower;
res(nx_lower:nx) = res_upper;  %mid_x is in both res_lower and res_upper
z = res;
%---- end of generate_non_uniform_grid


function z = generate_uniform_grid(nx, min_x, max_x)
dx = (max_x-min_x)/(nx-1);
i = 0:nx-1;
x = min_x + dx*i;
z = x;