%  ***************************************************************************
%  *   Copyright (C) 2009 by Aleksandar Mijatovic                            *
%  *   a.mijatovic@imperial.ac.uk                                            *
%  *                                                                         *
%  *   This program is free software; you can redistribute it and/or modify  *
%  *   it under the terms of the GNU General Public License as published by  *
%  *   the Free Software Foundation; either version 2 of the License, or     *
%  *   any later version.                                                    *
%  *                                                                         *
%  *   This program is distributed in the hope that it will be useful,       *
%  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
%  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
%  *   GNU General Public License for more details.                          *
%  *                                                                         *
%  *   You should have received a copy of the GNU General Public License     *
%  *   along with this program; if not, see                                  *
%  *   http://www.gnu.org/licenses/gpl-3.0.txt                               *
%  *   or write to the  Free Software Foundation, Inc.,                      *
%  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
%  ***************************************************************************

function main

tic 

%----- model parameters for diffusion  
%      dS/S = rate dt + vol (S/spotS)^beta dW

rate            = 0.1;               % risk free rate
vol             = 0.25;              % diffusion vol
beta            = -2;                % skewness
spotS           = 100;               % starting level


%----- state-space parameters  

N               = 400;               % number of points in the state-space of X
minS            = 0.1;               % smallest element in the grid
maxS            = 190;               % largest element in the grid


%----- contract details

T               = 0.5;                         % maturity
strikes         = spotS * [0.95:0.05:1.05];    % vector of strikes  
L               = 90;                          % lower barrier
U               = 120;                         % upper barrier


%--------------------------------------------------------------
% Markov generator algorithm
%--------------------------------------------------------------

% find the grid for the underlying

s_grid = generate_grid_for_double_barrier(N, minS, L, spotS,U, maxS, ...    
                                                 100, 1, ...
                                                 10, 10, ...
                                                 1, 100); 
                                            

%    compute the markov generator MG

MG = generate_diffusion_MG(s_grid, spotS, vol, rate, beta);


%    set up the call payoff

for y=1:size(strikes,2)
   payoff(:,y) = max(s_grid - strikes(y),0)';
end


%   compute the barrier generator and payoff

barrier_idx = find(L<s_grid & s_grid<U);
BarrierMG = MG(barrier_idx,barrier_idx);
bar_s_grid = s_grid(barrier_idx);
newspotS_index = find(bar_s_grid == spotS);


%   compute the barrier option prices

A = expm(T*BarrierMG);                      % exponentiate the barrier generator
barP = A(newspotS_index,:);
barPayoff = payoff(barrier_idx,:);
BarrierPrices = exp(-rate * T) * barP * barPayoff;

toc

fprintf('\n')
fprintf('---------------- CEV - Double Barrier Call Options -----------------\n')
fprintf('time to maturity:')
disp(T);
fprintf('strikes:')
disp(strikes);
fprintf('lower barrier, upper barrier:')
disp([L,U]);
fprintf('double barrier option prices:')
disp(BarrierPrices);
fprintf('---------------------------------------------------------------------\n')
end


